// Generated by CoffeeScript 1.7.1
(function() {
  var Condition, Liquid, Promise;

  Liquid = require("../liquid");

  Promise = require("bluebird");

  module.exports = Condition = (function() {
    Condition.operators = {
      '==': function(cond, left, right) {
        return cond.equalVariables(left, right);
      },
      'is': function(cond, left, right) {
        return cond.equalVariables(left, right);
      },
      '!=': function(cond, left, right) {
        return !cond.equalVariables(left, right);
      },
      '<>': function(cond, left, right) {
        return !cond.equalVariables(left, right);
      },
      'isnt': function(cond, left, right) {
        return cond.equalVariables(left, right);
      },
      '<': function(cond, left, right) {
        return left < right;
      },
      '>': function(cond, left, right) {
        return left > right;
      },
      '<=': function(cond, left, right) {
        return left <= right;
      },
      '>=': function(cond, left, right) {
        return left >= right;
      },
      'contains': function(cond, left, right) {
        return (left != null ? typeof left.indexOf === "function" ? left.indexOf(right) : void 0 : void 0) >= 0;
      }
    };

    function Condition(left, operator, right) {
      this.left = left;
      this.operator = operator;
      this.right = right;
      this.childRelation = null;
      this.childCondition = null;
    }

    Condition.prototype.evaluate = function(context) {
      var result;
      if (context == null) {
        context = new Liquid.Context();
      }
      result = this.interpretCondition(this.left, this.right, this.operator, context);
      switch (this.childRelation) {
        case "or":
          return Promise.cast(result).then((function(_this) {
            return function(result) {
              return result || _this.childCondition.evaluate(context);
            };
          })(this));
        case "and":
          return Promise.cast(result).then((function(_this) {
            return function(result) {
              return result && _this.childCondition.evaluate(context);
            };
          })(this));
        default:
          return result;
      }
    };

    Condition.prototype.or = function(childCondition) {
      this.childCondition = childCondition;
      return this.childRelation = "or";
    };

    Condition.prototype.and = function(childCondition) {
      this.childCondition = childCondition;
      return this.childRelation = "and";
    };

    Condition.prototype.attach = function(attachment) {
      this.attachment = attachment;
      return attachment;
    };

    Condition.prototype["else"] = function() {
      return false;
    };

    Condition.prototype.inspect = function() {
      var operands;
      operands = [this.left, this.operator, this.right].join(' ');
      return "<Condition [" + operands + "], attachment: " + this.attachment + ">";
    };

    Condition.prototype.equalVariables = function(left, right) {
      return left === right;
    };

    Condition.prototype.interpretCondition = function(left, right, op, context) {
      var operation;
      if (op == null) {
        return context.get(left);
      }
      operation = Condition.operators[op];
      if (operation == null) {
        throw new Error("Unknown operator " + op);
      }
      left = context.get(left);
      right = context.get(right);
      return Promise.join(left, right).spread((function(_this) {
        return function(left, right) {
          return operation(_this, left, right);
        };
      })(this));
    };

    return Condition;

  })();

}).call(this);

//# sourceMappingURL=condition.map
